//
// Note: This effect file works with EffectEdit.
//


// sparkle parameters
#define SPRINKLE    1.0
#define SCATTER     0.5

#define VOLUME_NOISE_SCALE  10

// scene info (EffectEdit specific controls)
string XFile = "teapot.x";
int    BCLR  = 0xff404080;

half time : TIME;

// textures
texture EnvironmentMap 
< 
    string type = "CUBE"; 
    string name = "Data\\Images\\Textures\\lobbycube.dds"; 
>;

// transformations
half4x3 WorldView  : WORLDVIEW;
half4x4 Projection : PROJECTION;

// light direction (view space)
half3 L < string UIDirectional = "Light Direction"; > = normalize(half3(-0.397f, -0.397f, 0.827f));

// light intensity
half4 I_a = { 0.3f, 0.3f, 0.3f, 1.0f };    // ambient
half4 I_d = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse
half4 I_s = { 0.7f, 0.7f, 0.7f, 1.0f };    // specular

// material reflectivity
half4 k_a : MATERIALAMBIENT = { 0.2f, 0.2f, 0.2f, 1.0f };    // ambient  (metal)
half4 k_d : MATERIALDIFFUSE = { 0.1f, 0.1f, 0.9f, 1.0f };    // diffuse  (metal)
half4 k_s : MATERIALSPECULAR = { 0.9f, 0.9f, 0.9f, 1.0f };    // specular (metal)
half4 k_r : MATERIALEMISSIVE = { 0.7f, 0.7f, 0.7f, 1.0f };    // specular (wax)



struct VSforFF_OUTPUT
{
    half4 Position   : POSITION;
    half3 TexCoord   : TEXCOORD; // envmap
};

// vertex shader whose output goes to the fixed function pipeline
VSforFF_OUTPUT VSforFF(
    half3 Position : POSITION,
    half3 Normal   : NORMAL,
    half2 TexCoord : TEXCOORD)
{
    VSforFF_OUTPUT Out = (VSforFF_OUTPUT)0;

    L = -L;

    half3 P = mul(half4(Position, 1), (half4x3)WorldView);   // position (view space)
    half3 N = normalize(mul(Normal, (half3x3)WorldView));     // normal (view space)
//    half3 R = normalize(2 * dot(N, L) * N - L);                // reflection vector (view space)
    half3 V = -normalize(P);                                   // view direction (view space)
    half3 G = normalize(2 * dot(N, V) * N - V);                // glance vector (view space)

    // position (projected)
    Out.Position = mul(half4(P, 1), Projection);             

    // environment cube map coordinates
    Out.TexCoord = half3(-G.x, G.y, -G.z);





    return Out;
}

// samplers
sampler Environment = sampler_state
{ 
    Texture = (EnvironmentMap);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};




// technique
technique TMetal
{
    pass P0
    {
        VertexShader = compile vs_1_1 VSforFF();
        PixelShader = NULL;

        // texture
        Texture[0] = (EnvironmentMap);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

        MinFilter[1] = LINEAR;
        MagFilter[1] = LINEAR;
        MipFilter[1] = LINEAR;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = SELECTARG1;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = DISABLE;

        ColorOp[1]   = DISABLE;
        ColorArg1[1] = TEXTURE;
        ColorArg2[1] = DIFFUSE;
        AlphaOp[1]   = DISABLE;

       AlphaBlendEnable = FALSE;
        SrcBlend         = ONE;
        DestBlend        = ZERO;
        CullMode         = CCW;
        DitherEnable = TRUE;
        
        SpecularEnable = false;
//        SpecularEnable = TRUE;
//        BlendOp = SUBTRACT;

    }
}

