//
// Wood Shader
// Copyright (c) Microsoft Corporation. All rights reserved.
//
// Note: This effect file works with EffectEdit.
//

//string XFile = "misc\\teapot.x";
string XFile = "teapot.x";
int    BCLR  = 0xff202080;

// transformations
float4x4 WorldViewProjection  : WORLDVIEWPROJECTION;
float4x3 WorldView  : WORLDVIEW;
float4x4 Projection : PROJECTION;

// light direction (view space)
float3 lightDir <  string UIDirectional = "Light Direction"; > = {0.577, -0.577, 0.577};

// light intensity
float4 I_a = { 0.3f, 0.3f, 0.3f, 1.0f };    // ambient
float4 I_d = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse
float4 I_s = { 0.6f, 0.6f, 0.6f, 1.0f };    // specular

// material reflectivity
float4 k_a = { 0.2f, 0.2f, 0.2f, 1.0f };    // ambient
float4 k_d = { 1.0f, 0.7f, 0.2f, 1.0f };    // diffuse
float4 k_s = { 1.0f, 1.0f, 1.0f, 1.0f };    // specular
float  n   = 64.0f;                         // power

float3 k_d_negative = { 0.35f, 0.4f, 1.0f }; // an inverse amount of the diffuse, for vs-only specular


#define DARK    0.3f
#define LIGHT   1.0f

// model dependent wood parameters
#define RINGSCALE   10.0f
#define POINTSCALE  2.0f
#define TURBULENCE  1.0f
#define VOLUME_SIZE 32


struct VSforFF_OUTPUT
{
    float4 Pos  : POSITION;
    float3 Diff : COLOR0;
};

VSforFF_OUTPUT VSforFF(    
    float3 Pos  : POSITION,
    float3 Norm : NORMAL)
{
    VSforFF_OUTPUT Out = (VSforFF_OUTPUT)0;

//    float3 L = -lightDir;
    Out.Pos = mul(float4(Pos, 1), (float4x4)WorldViewProjection);    // position (view space)
//    float3 N = normalize(mul(Norm, (float3x3)WorldView));   // normal (view space)
//    float3 R = normalize(2 * dot(N, L) * N - L);            // reflection vector (view space)
//    float3 V = -normalize(P);                               // view direction (view space)

//	Out.Diff = float3(1,0,1);
	Out.Diff = Norm;

//    Out.Pos  = mul(float4(P, 1), Projection);             // position (projected)
//    Out.Diff = 0.5*max(0, dot(N, L))  // diffuse + ambient
//             + 2.0*I_s * (min(k_s,k_d_negative)) * pow(max(0, dot(R, V)), n/4);   // specular
    return Out;
}




technique TWood_VS_1_1
{
    pass P0
    {
        VertexShader = compile vs_1_1 VSforFF();
        PixelShader = NULL;
    }
}
