//
// Metallic Flakes Shader
// Copyright (c) Microsoft Corporation. All rights reserved.
//
// Note: This effect file works with EffectEdit.
//
// NOTE:
// The metallic surface consists of 2 layers - 
// 1. a polished layer of wax on top (contributes a smooth specular reflection and an environment mapped reflection with a Fresnel term), and 
// 2. a blue metallic layer with a sprinkling of gold metallic flakes underneath


// sparkle parameters
#define SPRINKLE    1.0
#define SCATTER     0.5

#define VOLUME_NOISE_SCALE  10

// scene info (EffectEdit specific controls)
string XFile = "teapot.x";
int    BCLR  = 0xff404080;

texture tex : TEXTURE;
/*
< 
//    string type = "TEXTURE"; 
//    string function = "GenerateSparkle2D";
    string name = "noise.bmp";
    int width = 32;
    int height = 32;
//    int depth = 32; 
>;
*/
// transformations
half4x4 WorldIT : WorldInverseTranspose < string UIWidget="None"; >;
half4x4 WorldViewProj : WorldViewProjection < string UIWidget="None"; >;
half4x4 _world : _world < string UIWidget="None"; >;
half4x4 ViewI : ViewInverse < string UIWidget="None"; >;

// light direction (view space)
half3 L < string UIDirectional = "Light Direction"; > = normalize(half3(-0.397f, -0.397f, 0.827f));

// light intensity
half4 AmbiColor = { 0.3f, 0.3f, 0.3f, 1.0f };    // ambient
half4 LightColor = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse
half4 I_s = { 0.7f, 0.7f, 0.7f, 1.0f };    // specular

// material reflectivity
half4 k_a : MATERIALAMBIENT = { 0.2f, 0.2f, 0.2f, 1.0f };    // ambient  (metal)
half3 SurfColor : MATERIALDIFFUSE = { 0.1f, 0.1f, 0.9f };    // diffuse  (metal)
half4 k_s = { 0.5f, 0.5f, 0.5f, 1.0f };    // specular (metal)
half4 k_r = { 0.7f, 0.7f, 0.7f, 1.0f };    // specular (wax)
half SpecExpon = 16;


/* data from application vertex buffer */
struct appdata {
    half3 Position	: POSITION;
    half4 UV		: TEXCOORD0;
    half4 Normal	: NORMAL;
};
/* data passed from ps1 vertex shader to pixel shader */
struct vertexOutputPS1 {
    half4 HPosition	: POSITION;
    half4 TexCoord0	: TEXCOORD0;
    half4 diffCol	: COLOR0;
    half4 specCol	: COLOR1;
};
vertexOutputPS1 metalPVS(appdata IN)
{
    vertexOutputPS1 OUT;
    half3 Nn = normalize(mul(IN.Normal, WorldIT).xyz);
    half4 Po = half4(IN.Position.xyz,1);
    half3 Pw = mul(Po, _world).xyz;
    half3 LightPos = -L;
    half3 Ln = normalize(LightPos - Pw);
    half ldn = dot(Ln,Nn);
    half diffComp = max(0,ldn) * SurfColor;
    OUT.diffCol = half4((SurfColor*(diffComp*LightColor+AmbiColor)),1);
    OUT.TexCoord0 = IN.UV;
    half3 Vn = normalize(ViewI[3].xyz - Pw);
    half3 Hn = normalize(Vn + Ln);
    half hdn = pow(max(0,dot(Hn,Nn)),SpecExpon);
    OUT.specCol = half4((hdn * LightColor * SurfColor),1);
    OUT.HPosition = mul(Po, WorldViewProj);
    return OUT;
}

// technique
technique TMetallicFlakesNoPS
{
    pass P0
    {
		VertexShader = compile vs_1_1 metalPVS();
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = CCW;
		// no pixel shader needed
		SpecularEnable = true;
        Texture[0] = (tex);
		ColorOp[ 0 ]   = Modulate;
		ColorArg1[ 0 ] = Diffuse;
		ColorArg2[ 0 ] = Texture;
		AlphaArg1[ 0 ] = Diffuse;
		AlphaOp[ 0 ]   = SelectArg1;
    }
}

